@@ -99,6 +99,11 @@ def make_math_node(latex, docname, nowrap):
     node['number'] = None
     return node
 
+# To support any character in tacn, ... names.
+# see https://github.com/coq/coq/pull/13564
+def make_id(tag):
+    return tag.replace(" ", "-")
+
 class CoqObject(ObjectDescription):
     """A generic Coq object for Sphinx; all Coq objects are subclasses of this.
 
@@ -200,7 +205,7 @@ class CoqObject(ObjectDescription):
         names_in_subdomain[name] = (self.env.docname, self.objtype, target_id)
 
     def _target_id(self, name):
-        return make_target(self.objtype, nodes.make_id(name))
+        return make_target(self.objtype, make_id(name))
 
     def _add_target(self, signode, name):
         """Register a link target ‘name’, pointing to signode."""
@@ -210,6 +215,13 @@ class CoqObject(ObjectDescription):
             signode['names'].append(name)
             signode['first'] = (not self.names)
             self._record_name(name, targetid, signode)
+        else:
+            # todo: make the following a real error or warning
+            # todo: then maybe the above "if" is not needed
+            names_in_subdomain = self.subdomain_data()
+            if name in names_in_subdomain:
+                print("Duplicate", self.subdomain, "name: ", name)
+            # self._warn_if_duplicate_name(names_in_subdomain, name, signode)
         return targetid
 
     def _add_index_entry(self, name, target):
@@ -322,7 +334,7 @@ class VernacObject(NotationObject):
     annotation = "Command"
 
     def _name_from_signature(self, signature):
-        m = re.match(r"[a-zA-Z ]+", signature)
+        m = re.match(r"[a-zA-Z0-9_ ]+", signature)
         return m.group(0).strip() if m else None
 
 class VernacVariantObject(VernacObject):
@@ -333,7 +345,7 @@ class VernacVariantObject(VernacObject):
        .. cmd:: Axiom @ident : @term.
 
           This command links :token:`term` to the name :token:`term` as its specification in
-          the global context. The fact asserted by :token:`term` is thus assumed as a
+          the global environment. The fact asserted by :token:`term` is thus assumed as a
           postulate.
 
           .. cmdv:: Parameter @ident : @term.
@@ -351,7 +363,7 @@ class TacticObject(NotationObject):
 
     Example::
 
-       .. tacn:: do @num @expr
+       .. tacn:: do @natural @expr
 
           :token:`expr` is evaluated to ``v`` which must be a tactic value. …
     """
@@ -401,7 +413,7 @@ class OptionObject(NotationObject):
 
     Example::
 
-       .. opt:: Hyps Limit @num
+       .. opt:: Hyps Limit @natural
           :name Hyps Limit
 
           Controls the maximum number of hypotheses displayed in goals after
@@ -452,7 +464,7 @@ class ProductionObject(CoqObject):
 
     Example::
 
-        .. prodn:: occ_switch ::= { {? {| + | - } } {* @num } }
+        .. prodn:: occ_switch ::= { {? {| + | - } } {* @natural } }
         term += let: @pattern := @term in @term
         | second_production
 
@@ -494,14 +506,18 @@ class ProductionObject(CoqObject):
                 loc = os.path.basename(get_node_location(signode))
                 raise ExtensionError(ProductionObject.SIG_ERROR.format(loc, signature))
 
-        self.signatures.append((lhs, op, rhs))
+        parts = rhs.split("   ", maxsplit=1)
+        rhs = parts[0].strip()
+        tag = parts[1].strip() if len(parts) == 2 else ""
+
+        self.signatures.append((lhs, op, rhs, tag))
         return [('token', lhs)] if op == '::=' else None
 
     def _add_index_entry(self, name, target):
         pass
 
     def _target_id(self, name):
-        return 'grammar-token-{}'.format(nodes.make_id(name[1]))
+        return make_id('grammar-token-{}'.format(name[1]))
 
     def _record_name(self, name, targetid, signode):
         env = self.state.document.settings.env
@@ -515,7 +531,7 @@ class ProductionObject(CoqObject):
 
         table = nodes.inline(classes=['prodn-table'])
         tgroup = nodes.inline(classes=['prodn-column-group'])
-        for _ in range(3):
+        for _ in range(4):
             tgroup += nodes.inline(classes=['prodn-column'])
         table += tgroup
         tbody = nodes.inline(classes=['prodn-row-group'])
@@ -523,13 +539,13 @@ class ProductionObject(CoqObject):
 
         # create rows
         for signature in self.signatures:
-            lhs, op, rhs = signature
+            lhs, op, rhs, tag = signature
             position = self.state_machine.get_source_and_line(self.lineno)
 
             row = nodes.inline(classes=['prodn-row'])
             entry = nodes.inline(classes=['prodn-cell-nonterminal'])
             if lhs != "":
-                target_name = 'grammar-token-' + nodes.make_id(lhs)
+                target_name = make_id('grammar-token-' + lhs)
                 target = nodes.target('', '', ids=[target_name], names=[target_name])
                 # putting prodn-target on the target node won't appear in the tex file
                 inline = nodes.inline(classes=['prodn-target'])
@@ -548,6 +564,10 @@ class ProductionObject(CoqObject):
             entry += notation_to_sphinx(rhs, *position)
             row += entry
 
+            entry = nodes.inline(classes=['prodn-cell-tag'])
+            entry += nodes.literal(tag, tag)
+            row += entry
+
             tbody += row
 
         return [indexnode, table] # only this node goes into the doc
@@ -854,7 +874,7 @@ class InferenceDirective(Directive):
         docname = self.state.document.settings.env.docname
         math_node = make_math_node(latex, docname, nowrap=False)
 
-        tid = nodes.make_id(title)
+        tid = make_id(title)
         target = nodes.target('', '', ids=['inference-' + tid])
         self.state.document.note_explicit_target(target)
 
@@ -918,10 +938,11 @@ class CoqtopBlocksTransform(Transform):
 
     @staticmethod
     def split_lines(source):
-        r"""Split Coq input in chunks
+        r"""Split Coq input into chunks, which may include single- or
+        multi-line comments.  Nested comments are not supported.
 
         A chunk is a minimal sequence of consecutive lines of the input that
-        ends with a '.'
+        ends with a '.' or '*)'
 
         >>> split_lines('A.\nB.''')
         ['A.', 'B.']
@@ -942,8 +963,14 @@ class CoqtopBlocksTransform(Transform):
 
         >>> split_lines('SearchHead le.\nSearchHead (@eq bool).')
         ['SearchHead le.', 'SearchHead (@eq bool).']
+
+        >>> split_lines("(* *) x. (* *)\ny.\n")
+        ['(* *) x. (* *)', 'y.']
+
+        >>> split_lines("(* *) x (* \n *)\ny.\n")
+        ['(* *) x (* \n *)', 'y.']
         """
-        return re.split(r"(?<=(?<!\.)\.)\n", source.strip())
+        return re.split(r"(?:(?<=(?<!\.)\.)|(?<=\*\)))\n", source.strip())
 
     @staticmethod
     def parse_options(node):
@@ -1019,7 +1046,11 @@ class CoqtopBlocksTransform(Transform):
         if options['warn']:
             repl.sendone('Set Warnings "default".')
         for sentence in self.split_lines(node.rawsource):
-            pairs.append((sentence, repl.sendone(sentence)))
+            comment = re.compile(r"\s*\(\*.*?\*\)\s*", re.DOTALL)
+            wo_comments = re.sub(comment, "", sentence)
+            has_tac = wo_comments != "" and not wo_comments.isspace()
+            output = repl.sendone(sentence) if has_tac else ""
+            pairs.append((sentence, output))
         if options['abort']:
             repl.sendone('Abort All.')
         if options['fail']:
@@ -1032,9 +1063,10 @@ class CoqtopBlocksTransform(Transform):
             # Use Coqdoc to highlight input
             in_chunks = highlight_using_coqdoc(sentence)
             dli += nodes.term(sentence, '', *in_chunks, classes=self.block_classes(options['input']))
-            # Parse ANSI sequences to highlight output
-            out_chunks = AnsiColorsParser().colorize_str(output)
-            dli += nodes.definition(output, *out_chunks, classes=self.block_classes(options['output'], output))
+            if output:
+                # Parse ANSI sequences to highlight output
+                out_chunks = AnsiColorsParser().colorize_str(output)
+                dli += nodes.definition(output, *out_chunks, classes=self.block_classes(options['output'], output))
         node.clear()
         node.rawsource = self.make_rawsource(pairs, options['input'], options['output'])
         node['classes'].extend(self.block_classes(options['input'] or options['output']))
@@ -1151,17 +1183,19 @@ class StdGlossaryIndex(Index):
     name, localname, shortname = "glossindex", "Glossary", "terms"
 
     def generate(self, docnames=None):
-        content = defaultdict(list)
+        def ci_sort(entry):
+            ((type, itemname), (docname, anchor)) = entry
+            return itemname.lower()
 
-        for ((type, itemname), (docname, anchor)) in self.domain.data['objects'].items():
+        content = defaultdict(list)
+        for ((type, itemname), (docname, anchor)) in sorted(self.domain.data['objects'].items(), key=ci_sort):
             if type == 'term':
                 entries = content[itemname[0].lower()]
                 entries.append([itemname, 0, docname, anchor, '', '', ''])
-        content = sorted(content.items())
-        return content, False
+        return content.items(), False
 
 def GrammarProductionRole(typ, rawtext, text, lineno, inliner, options={}, content=[]):
-    """A grammar production not included in a ``productionlist`` directive.
+    """A grammar production not included in a ``prodn`` directive.
 
     Useful to informally introduce a production, as part of running text.
 
@@ -1169,14 +1203,12 @@ def GrammarProductionRole(typ, rawtext, text, lineno, inliner, options={}, conte
 
        :production:`string` indicates a quoted string.
 
-    You're not likely to use this role very commonly; instead, use a
-    `production list
-    <http://www.sphinx-doc.org/en/stable/markup/para.html#directive-productionlist>`_
-    and reference its tokens using ``:token:`…```.
+    You're not likely to use this role very commonly; instead, use a ``prodn``
+    directive and reference its tokens using ``:token:`…```.
     """
     #pylint: disable=dangerous-default-value, unused-argument
     env = inliner.document.settings.env
-    targetid = nodes.make_id('grammar-token-{}'.format(text))
+    targetid = make_id('grammar-token-{}'.format(text))
     target = nodes.target('', '', ids=[targetid])
     inliner.document.note_explicit_target(target)
     code = nodes.literal(rawtext, text, role=typ.lower())
@@ -1215,7 +1247,7 @@ def GlossaryDefRole(typ, rawtext, text, lineno, inliner, options={}, content=[])
         msg = MSG.format(term, env.doc2path(std[key][0]))
         inliner.document.reporter.warning(msg, line=lineno)
 
-    targetid = nodes.make_id('term-{}'.format(term))
+    targetid = make_id('term-{}'.format(term))
     std[key] = (env.docname, targetid)
     target = nodes.target('', '', ids=[targetid], names=[term])
     inliner.document.note_explicit_target(target)
@@ -1418,11 +1450,11 @@ def setup(app):
     app.connect('doctree-resolved', CoqtopBlocksTransform.merge_consecutive_coqtop_blocks)
 
     # Add extra styles
-    app.add_stylesheet("ansi.css")
-    app.add_stylesheet("coqdoc.css")
-    app.add_javascript("notations.js")
-    app.add_stylesheet("notations.css")
-    app.add_stylesheet("pre-text.css")
+    app.add_css_file("ansi.css")
+    app.add_css_file("coqdoc.css")
+    app.add_js_file("notations.js")
+    app.add_css_file("notations.css")
+    app.add_css_file("pre-text.css")
 
     # Tell Sphinx about extra settings
     app.add_config_value("report_undocumented_coq_objects", None, 'env')
