@@ -26,7 +26,10 @@ let safe_flags oracle = {
   enable_VM = true;
   enable_native_compiler = true;
   indices_matter = true;
+  impredicative_set = false;
+  sprop_allowed = true;
   cumulative_sprop = false;
+  allow_uip = false;
 }
 
 (** {6 Arities } *)
@@ -54,14 +57,14 @@ let hcons_template_universe ar =
     template_context = Univ.hcons_universe_context_set ar.template_context }
 
 let universes_context = function
-  | Monomorphic _ -> Univ.AUContext.empty
+  | Monomorphic -> Univ.AbstractContext.empty
   | Polymorphic ctx -> ctx
 
 let abstract_universes = function
-  | Entries.Monomorphic_entry ctx ->
-    Univ.empty_level_subst, Monomorphic ctx
-  | Entries.Polymorphic_entry (nas, ctx) ->
-    let (inst, auctx) = Univ.abstract_universes nas ctx in
+  | Entries.Monomorphic_entry ->
+    Univ.empty_level_subst, Monomorphic
+  | Entries.Polymorphic_entry uctx ->
+    let (inst, auctx) = Univ.abstract_universes uctx in
     let inst = Univ.make_instance_subst inst in
     (inst, Polymorphic auctx)
 
@@ -69,7 +72,7 @@ let abstract_universes = function
 
 let constant_is_polymorphic cb =
   match cb.const_universes with
-  | Monomorphic _ -> false
+  | Monomorphic -> false
   | Polymorphic _ -> true
 
 
@@ -99,7 +102,7 @@ let subst_const_type sub arity =
     at least for Def due to the delayed substitution [subst_constr_subst]. *)
 let subst_const_def sub def = match def with
   | Undef _ | Primitive _ -> def
-  | Def c -> Def (subst_constr sub c)
+  | Def c -> Def (subst_mps sub c)
   | OpaqueDef o -> OpaqueDef (Opaqueproof.subst_opaque sub o)
 
 let subst_const_body sub cb =
@@ -115,7 +118,7 @@ let subst_const_body sub cb =
         const_body = body';
         const_type = type';
         const_body_code =
-          Option.map (Cemitcodes.subst_to_patch_subst sub) cb.const_body_code;
+          Option.map (Vmemitcodes.subst_body_code sub) cb.const_body_code;
         const_universes = cb.const_universes;
         const_relevance = cb.const_relevance;
         const_inline_code = cb.const_inline_code;
@@ -136,14 +139,12 @@ let hcons_const_def = function
   | Undef inl -> Undef inl
   | Primitive p -> Primitive p
   | Def l_constr ->
-    let constr = force_constr l_constr in
-    Def (from_val (Constr.hcons constr))
+    Def (Constr.hcons l_constr)
   | OpaqueDef _ as x -> x (* hashconsed when turned indirect *)
 
 let hcons_universes cbu =
   match cbu with
-  | Monomorphic ctx ->
-    Monomorphic (Univ.hcons_universe_context_set ctx)
+  | Monomorphic -> Monomorphic
   | Polymorphic ctx ->
     Polymorphic (Univ.hcons_abstract_universe_context ctx)
 
@@ -155,21 +156,47 @@ let hcons_const_body cb =
   }
 
 (** {6 Inductive types } *)
+let eq_nested_type t1 t2 = match t1, t2 with
+| NestedInd ind1, NestedInd ind2 -> Names.Ind.CanOrd.equal ind1 ind2
+| NestedInd _, _ -> false
+| NestedPrimitive c1, NestedPrimitive c2 -> Names.Constant.CanOrd.equal c1 c2
+| NestedPrimitive _, _ -> false
 
 let eq_recarg r1 r2 = match r1, r2 with
 | Norec, Norec -> true
-| Mrec i1, Mrec i2 -> Names.eq_ind i1 i2
-| Imbr i1, Imbr i2 -> Names.eq_ind i1 i2
-| _ -> false
+| Norec, _ -> false
+| Mrec i1, Mrec i2 -> Names.Ind.CanOrd.equal i1 i2
+| Mrec _, _ -> false
+| Nested ty1, Nested ty2 -> eq_nested_type ty1 ty2
+| Nested _, _ -> false
+
+let pp_recarg = let open Pp in function
+  | Declarations.Norec -> str "Norec"
+  | Declarations.Mrec (mind,i) ->
+     str "Mrec[" ++ Names.MutInd.print mind ++ pr_comma () ++ int i ++ str "]"
+  | Declarations.(Nested (NestedInd (mind,i))) ->
+     str "Nested[" ++ Names.MutInd.print mind ++ pr_comma () ++ int i ++ str "]"
+  | Declarations.(Nested (NestedPrimitive c)) ->
+     str "Nested[" ++ Names.Constant.print c ++ str "]"
+
+let pp_wf_paths x = Rtree.pp_tree pp_recarg x
+
+let subst_nested_type sub ty = match ty with
+| NestedInd (kn,i) ->
+  let kn' = subst_mind sub kn in
+  if kn==kn' then ty else NestedInd (kn',i)
+| NestedPrimitive c ->
+  let c',_ = subst_con sub c in
+  if c==c' then ty else NestedPrimitive c'
 
 let subst_recarg sub r = match r with
   | Norec -> r
   | Mrec (kn,i) ->
     let kn' = subst_mind sub kn in
     if kn==kn' then r else Mrec (kn',i)
-  | Imbr (kn,i) ->
-    let kn' = subst_mind sub kn in
-    if kn==kn' then r else Imbr (kn',i)
+  | Nested ty ->
+    let ty' = subst_nested_type sub ty in
+    if ty==ty' then r else Nested ty'
 
 let mk_norec = Rtree.mk_node Norec [||]
 
@@ -261,7 +288,7 @@ let inductive_polymorphic_context mib =
 
 let inductive_is_polymorphic mib =
   match mib.mind_universes with
-  | Monomorphic _ -> false
+  | Monomorphic -> false
   | Polymorphic _ctx -> true
 
 let inductive_is_cumulative mib =
