@@ -9,6 +9,7 @@
 (************************************************************************)
 
 open Names
+open Declarations
 
 (** This module defines the global environment of Coq. The functions
    below are exactly the same as the ones in [Safe_typing], operating on
@@ -29,13 +30,14 @@ val named_context : unit -> Constr.named_context
 (** {6 Enriching the global environment } *)
 
 (** Changing the (im)predicativity of the system *)
-val set_engagement : Declarations.engagement -> unit
+val set_impredicative_set : bool -> unit
+
 val set_indices_matter : bool -> unit
-val set_typing_flags : Declarations.typing_flags -> unit
+val set_typing_flags : typing_flags -> unit
 val set_check_guarded : bool -> unit
 val set_check_positive : bool -> unit
 val set_check_universes : bool -> unit
-val typing_flags : unit -> Declarations.typing_flags
+val typing_flags : unit -> typing_flags
 val set_cumulative_sprop : bool -> unit
 val is_cumulative_sprop : unit -> bool
 val set_allow_sprop : bool -> unit
@@ -45,33 +47,36 @@ val sprop_allowed : unit -> bool
 
 val push_named_assum : (Id.t * Constr.types) -> unit
 val push_named_def   : (Id.t * Entries.section_def_entry) -> unit
-val push_section_context : (Name.t array * Univ.UContext.t) -> unit
+val push_section_context : Univ.UContext.t -> unit
 
 val export_private_constants :
   Safe_typing.private_constants ->
   Safe_typing.exported_private_constant list
 
 val add_constant :
+  ?typing_flags:typing_flags ->
   Id.t -> Safe_typing.global_declaration -> Constant.t
+val fill_opaque : Safe_typing.opaque_certificate -> unit
 val add_private_constant :
-  Id.t -> Safe_typing.side_effect_declaration -> Constant.t * Safe_typing.private_constants
+  Id.t -> Univ.ContextSet.t -> Safe_typing.side_effect_declaration -> Constant.t * Safe_typing.private_constants
 val add_mind :
+  ?typing_flags:typing_flags ->
   Id.t -> Entries.mutual_inductive_entry -> MutInd.t
 
 (** Extra universe constraints *)
-val add_constraints : Univ.Constraint.t -> unit
+val add_constraints : Univ.Constraints.t -> unit
 
 val push_context_set : strict:bool -> Univ.ContextSet.t -> unit
 
 (** Non-interactive modules and module types *)
 
 val add_module :
-  Id.t -> Entries.module_entry -> Declarations.inline ->
+  Id.t -> Entries.module_entry -> inline ->
     ModPath.t * Mod_subst.delta_resolver
 val add_modtype :
-  Id.t -> Entries.module_type_entry -> Declarations.inline -> ModPath.t
+  Id.t -> Entries.module_type_entry -> inline -> ModPath.t
 val add_include :
-  Entries.module_struct_entry -> bool -> Declarations.inline ->
+  Entries.module_struct_entry -> bool -> inline ->
     Mod_subst.delta_resolver
 
 (** Sections *)
@@ -91,50 +96,54 @@ val start_module : Id.t -> ModPath.t
 val start_modtype : Id.t -> ModPath.t
 
 val end_module : Summary.frozen -> Id.t ->
-  (Entries.module_struct_entry * Declarations.inline) option ->
+  (Entries.module_struct_entry * inline) option ->
     ModPath.t * MBId.t list * Mod_subst.delta_resolver
 
 val end_modtype : Summary.frozen -> Id.t -> ModPath.t * MBId.t list
 
 val add_module_parameter :
-  MBId.t -> Entries.module_struct_entry -> Declarations.inline ->
+  MBId.t -> Entries.module_struct_entry -> inline ->
     Mod_subst.delta_resolver
 
 (** {6 Queries in the global environment } *)
 
 val lookup_named     : variable -> Constr.named_declaration
-val lookup_constant  : Constant.t -> Opaqueproof.opaque Declarations.constant_body
+val lookup_constant  : Constant.t -> constant_body
 val lookup_inductive : inductive ->
-  Declarations.mutual_inductive_body * Declarations.one_inductive_body
+  mutual_inductive_body * one_inductive_body
 val lookup_pinductive : Constr.pinductive ->
-  Declarations.mutual_inductive_body * Declarations.one_inductive_body
-val lookup_mind      : MutInd.t -> Declarations.mutual_inductive_body
-val lookup_module    : ModPath.t -> Declarations.module_body
-val lookup_modtype   : ModPath.t -> Declarations.module_type_body
+  mutual_inductive_body * one_inductive_body
+val lookup_mind      : MutInd.t -> mutual_inductive_body
+val lookup_module    : ModPath.t -> module_body
+val lookup_modtype   : ModPath.t -> module_type_body
 val exists_objlabel  : Label.t -> bool
 
 val constant_of_delta_kn : KerName.t -> Constant.t
 val mind_of_delta_kn : KerName.t -> MutInd.t
 
-val opaque_tables : unit -> Opaqueproof.opaquetab
+type indirect_accessor = {
+  access_proof : cooking_info Opaqueproof.opaque -> (Constr.t * unit Opaqueproof.delayed_universes) option;
+}
 
-val body_of_constant : Opaqueproof.indirect_accessor -> Constant.t ->
-  (Constr.constr * unit Opaqueproof.delayed_universes * Univ.AUContext.t) option
+val force_proof : indirect_accessor -> cooking_info Opaqueproof.opaque -> Constr.t * unit Opaqueproof.delayed_universes
+
+val body_of_constant : indirect_accessor -> Constant.t ->
+  (Constr.constr * unit Opaqueproof.delayed_universes * Univ.AbstractContext.t) option
 (** Returns the body of the constant if it has any, and the polymorphic context
     it lives in. For monomorphic constant, the latter is empty, and for
     polymorphic constants, the term contains De Bruijn universe variables that
     need to be instantiated. *)
 
-val body_of_constant_body : Opaqueproof.indirect_accessor ->
-  Opaqueproof.opaque Declarations.constant_body ->
-    (Constr.constr * unit Opaqueproof.delayed_universes * Univ.AUContext.t) option
-(** Same as {!body_of_constant} but on {!Declarations.constant_body}. *)
+val body_of_constant_body : indirect_accessor ->
+  constant_body ->
+    (Constr.constr * unit Opaqueproof.delayed_universes * Univ.AbstractContext.t) option
+(** Same as {!body_of_constant} but on {!constant_body}. *)
 
 (** {6 Compiled libraries } *)
 
 val start_library : DirPath.t -> ModPath.t
-val export : ?except:Future.UUIDSet.t -> output_native_objects:bool -> DirPath.t ->
-  ModPath.t * Safe_typing.compiled_library * Safe_typing.native_library
+val export : output_native_objects:bool -> DirPath.t ->
+  ModPath.t * Safe_typing.compiled_library * Nativelib.native_library
 val import :
   Safe_typing.compiled_library -> Univ.ContextSet.t -> Safe_typing.vodigest ->
   ModPath.t
@@ -146,7 +155,6 @@ val import :
 
 val env_of_context : Environ.named_context_val -> Environ.env
 
-val join_safe_environment : ?except:Future.UUIDSet.t -> unit -> unit
 val is_joined_environment : unit -> bool
 
 val is_polymorphic : GlobRef.t -> bool
